//---------------------------------------------------------------------------
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
// PURPOSE.
//
//                                                    Albert Lee 2003/12/09
//---------------------------------------------------------------------------
#include <windows.h>
#include <condefs.h>
#include "TZlib.h"

#pragma link "ZLib.LIB"
//---------------------------------------------------------------------------
__fastcall TZLibBase::TZLibBase()
  : TObject()
{
	ZLibRec = new z_stream();
	ZLibRec->zalloc = 0;
	ZLibRec->zfree = 0;
	ZLibRec->opaque = 0;
	Length = 0;
}
//---------------------------------------------------------------------------
__fastcall TZLibBase::~TZLibBase()
{
	delete ZLibRec;
}
//---------------------------------------------------------------------------
int __fastcall TZLibBase::LoadInput(TStream* InStream, z_stream *ZLibRec)
{
	if (ZLibRec->avail_in == 0)
	{
		ZLibRec->next_in = InputBuffer;
		ZLibRec->avail_in = InStream->Read(InputBuffer, Z_BUFFER_SIZE);
	}
	return ZLibRec->avail_in;
}
//---------------------------------------------------------------------------
int __fastcall TZLibBase::FlushOutput(TStream* OutStream, z_stream *ZLibRec)
{
	unsigned int FlashSize = Z_BUFFER_SIZE - ZLibRec->avail_out;

	if (FlashSize)
	{
	  OutStream->Write(OutputBuffer, FlashSize);
	  ZLibRec->next_out = OutputBuffer;
	  ZLibRec->avail_out = Z_BUFFER_SIZE;
	}
	return FlashSize;
}
//---------------------------------------------------------------------------
#ifdef	PROCESS_MSG_ON_ZLIB
void __fastcall TZLibBase::ZLibMsgHandler(void)
{
	MSG message;

	while (PeekMessage(&message, NULL, 0, 0, PM_REMOVE))
	{
		TranslateMessage(&message);
		DispatchMessage(&message);
	}
}
#endif	//PROCESS_MSG_ON_ZLIB
//---------------------------------------------------------------------------
int __fastcall TZLibBase::Percent(void)
{
	if (Length == 0)
	{	return 100;	}
	else if (Length > 10000000L)
	{	return (ZLibRec->total_in /(Length / 100));	}
	else
	{	return (ZLibRec->total_in * 100 / Length);	}
}
//---------------------------------------------------------------------------
const char* __fastcall TZLibBase::ZLibVersion(void)
{
#ifdef	_DEBUG
	char *result;
	result = (char *)zlibVersion();
	return((const char*)result);
#else	//_DEBUG
	return(zlibVersion());
#endif	//_DEBUG
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
__fastcall TZLibCompressionStream::TZLibCompressionStream()
  : TZLibBase()
{
}
//---------------------------------------------------------------------------
__fastcall TZLibCompressionStream::~TZLibCompressionStream()
{
}
//---------------------------------------------------------------------------
int __fastcall TZLibCompressionStream::Compress(TStream *InStream, TStream *OutStream,
		int compresslevel)
{
	int result = Z_OK;

	OutStream->Size = 0;

	//check the compresslevel
	if (compresslevel == 0)
	{	//disable compress if compresslevel == 0
		OutStream->CopyFrom(InStream, 0);		//copy full stream
		InStream->Position = 0;
		OutStream->Position = 0;

		return (result);
	}
	if (compresslevel > 9)
	{	compresslevel = 9;  }
	else if (compresslevel < 0)
	{	compresslevel = Z_BEST_SPEED;	}		//Z_BEST_SPEED = 1

	//prepare environment
	InStream->Position = 0;
//	OutStream->Position = 0;

	ZLibRec->avail_in = 0;
	ZLibRec->avail_out = Z_BUFFER_SIZE;
	ZLibRec->next_out = OutputBuffer;
	UserAbort = false;

	Length = InStream->Size;
	result = deflateInit(ZLibRec, compresslevel);  // zlib call
	if (result != Z_OK)
	{	return (result);	}

	while (!UserAbort && LoadInput(InStream, ZLibRec))
	{
		ZLibMsgHandler();
		result = deflate(ZLibRec, Z_NO_FLUSH);	// zlib call

#ifdef	ENLARGE_STREAM_ON_PROCESS
		unsigned int AdjustSize;

		if ((OutStream->Size % Z_ALLOCATE_BLOCK))
		{
			AdjustSize = (OutStream->Size + Z_ALLOCATE_BLOCK) -
				(OutStream->Size % Z_ALLOCATE_BLOCK);
			OutStream->Size = AdjustSize;
		}
#endif;	ENLARGE_STREAM_ON_PROCESS

		if (ZLibRec->avail_out == 0)			//flash if buffer full
		{	FlushOutput(OutStream, ZLibRec);	}
		if (result != Z_OK)
		{	break;	}
//		Progress(Percent());
	}

	while (!UserAbort)
	{
		ZLibMsgHandler();
		result = deflate(ZLibRec, Z_FINISH);	// zlib call
		if ((FlushOutput(OutStream, ZLibRec) == 0) || (result != Z_OK))
		{	break;	}
	}

//	Progress(Percent());

	deflateEnd(ZLibRec);	// zlib call

#ifdef	ENLARGE_STREAM_ON_PROCESS
	OutStream->Size = ZLibRec->total_out;		//Adjust final size for stream
#endif	//ENLARGE_STREAM_ON_PROCESS	

	if (UserAbort)
	{	result = Z_USER_ABORT;	}
	else if (result == Z_STREAM_END)
	{	result = Z_OK;	}

	if (result != Z_OK)
	{	OutStream->Size = 0;	}				//clear OutStream if error
	else 
	{	OutStream->Position = 0;	}

	InStream->Position = 0;

	return (result);
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
__fastcall TZLibDecompressionStream::TZLibDecompressionStream()
  : TZLibBase()
{
}
//---------------------------------------------------------------------------
__fastcall TZLibDecompressionStream::~TZLibDecompressionStream()
{
}
//---------------------------------------------------------------------------
int __fastcall TZLibDecompressionStream::Decompress(TStream *InStream,
	TStream *OutStream)
{
	int result = Z_OK;
	ZLibRec->avail_in = 0;
	ZLibRec->avail_out = Z_BUFFER_SIZE;
	ZLibRec->next_in = InputBuffer;
	ZLibRec->next_out = OutputBuffer;
	UserAbort = false;

	InStream->Position = 0;
	OutStream->Size = 0;
//	OutStream->Position = 0;

	Length = InStream->Size;
	result = inflateInit(ZLibRec);   		   	// zlib call
	if (result != Z_OK)
	{	return (result);	}

	while (!UserAbort && LoadInput(InStream, ZLibRec))
	{
		ZLibMsgHandler();
		result = inflate(ZLibRec, Z_NO_FLUSH);	// zlib call

#ifdef	ENLARGE_STREAM_ON_PROCESS
		unsigned int AdjustSize;

		if ((OutStream->Size % Z_ALLOCATE_BLOCK))
		{
			AdjustSize = (OutStream->Size + Z_ALLOCATE_BLOCK) -
				(OutStream->Size % Z_ALLOCATE_BLOCK);
			OutStream->Size = AdjustSize;
		}
#endif;	ENLARGE_STREAM_ON_PROCESS

		if (ZLibRec->avail_out == 0)
		{	FlushOutput(OutStream, ZLibRec);	}
		if (result != Z_OK)
		{	break;	}
//		Progress(Percent());
	}

	while (!UserAbort)
	{
		ZLibMsgHandler();
		result = inflate(ZLibRec, Z_FINISH);	// zlib call
		if ((FlushOutput(OutStream, ZLibRec) == 0) || (result != Z_OK))
		{	break;	}
	}

//	Progress(Percent());

	inflateEnd(ZLibRec);						// zlib call

#ifdef	ENLARGE_STREAM_ON_PROCESS
	OutStream->Size = ZLibRec->total_out;		//Adjust final size for stream
#endif	//ENLARGE_STREAM_ON_PROCESS	

	if (UserAbort)
	{	result = Z_USER_ABORT;	}
	else if (result == Z_STREAM_END)
	{	result = Z_OK;	}

	if (result != Z_OK)
	{	OutStream->Size = 0;	}				//clear OutStream if error
	else 
	{	OutStream->Position = 0;	}

	InStream->Position = 0;

	return (result);
}
//---------------------------------------------------------------------------
